import path from 'path';
import crypto from 'crypto';

const HASH_LENGTH = 7;
/**
 * Get content byte size
 */
function getByteSize(content) {
    if (typeof content === 'string') {
        return Buffer.from(content).length;
    }
    return content?.length || 0;
}
/**
 * Generate a 7 chars hash from a filepath
 */
function getHash(filepath) {
    const digest = crypto.createHash('sha256');
    return digest.update(Buffer.from(filepath)).digest('hex').substr(0, HASH_LENGTH);
}
function getChunkId(chunk) {
    let value = chunk.name;
    // Use entry module relative path
    if (chunk.moduleIds?.length > 0) {
        const absoluteModulePath = chunk.moduleIds[chunk.moduleIds.length - 1];
        value = path.relative(process.cwd(), absoluteModulePath);
    }
    return getHash([chunk, value].join('-'));
}

/**
 * Recursivily check if a chunk is async based on the chunks issuers
 */
const lookupChunkAsync = (chunk, chunksIssuers) => {
    if (chunk.isDynamicEntry) {
        return true;
    }
    if (chunk.isEntry) {
        return false;
    }
    const chunkIssuers = chunksIssuers[chunk.fileName];
    /**
     * A sync chunk without issuer chunks, is sync
     */
    if (!chunkIssuers) {
        return false;
    }
    const syncChunksIssuers = chunkIssuers.filter((chunkIssuer) => {
        return chunkIssuer.isDynamicEntry === false;
    });
    /**
     * A sync chunk with all the chunk issuer async, is async
     */
    if (syncChunksIssuers.length === 0) {
        return true;
    }
    /**
     * Recursively lookup for sync loads on the 2nd level issuers
     * - if at least one issuer is sync, the chunk is sync
     * - if none of the issuers are sync, the chunk is async
     */
    let isAsync = true;
    for (let i = 0; i < syncChunksIssuers.length && isAsync; i++) {
        isAsync = lookupChunkAsync(syncChunksIssuers[i], chunksIssuers);
    }
    return isAsync;
};
/**
 * Store transformed sources
 */
class TransformSources {
    constructor() {
        this.entries = {};
    }
    entries = {};
    push(id, source) {
        this.entries[id] = source;
    }
    /**
     * Get asset source
     */
    getByAsset = (asset) => {
        return this.entries[asset.name];
    };
    /**
     * Get chunk source
     */
    getByChunk = (chunk) => {
        return this.entries[chunk.id];
    };
    /**
     * Get module source
     */
    getByModule = (module) => {
        return this.entries[module.name];
    };
}
const defaultTransform = (stats) => stats;
const bundleToWebpackStats = (bundle, pluginOptions) => {
    const options = { moduleOriginalSize: false, ...pluginOptions };
    const { moduleOriginalSize, transform = defaultTransform } = options;
    const assets = [];
    const chunks = [];
    const moduleByFileName = {};
    const sources = new TransformSources();
    const chunksIssuers = {};
    const entries = Object.values(bundle);
    // Collect metadata
    entries.forEach((entry) => {
        if (entry.type === 'chunk') {
            entry.imports?.forEach((chunkDependency) => {
                if (!chunksIssuers[chunkDependency]) {
                    chunksIssuers[chunkDependency] = [];
                }
                chunksIssuers[chunkDependency].push(entry);
            });
        }
    });
    // Process data
    entries.forEach((entry) => {
        if (entry.type === 'chunk') {
            assets.push({
                name: entry.fileName,
                size: getByteSize(entry.code),
            });
            sources.push(entry.fileName, entry);
            const chunkId = getChunkId(entry);
            const chunkAsync = lookupChunkAsync(entry, chunksIssuers);
            chunks.push({
                id: chunkId,
                entry: entry.isEntry,
                initial: !chunkAsync,
                files: [entry.fileName],
                names: [entry.name],
            });
            sources.push(chunkId, entry);
            Object.entries(entry.modules).forEach(([modulePath, moduleInfo]) => {
                // Remove unexpected rollup null prefix
                const normalizedModulePath = modulePath.replace('\u0000', '');
                const relativeModulePath = path.relative(process.cwd(), normalizedModulePath);
                // Match webpack output - add current directory prefix for child modules
                const relativeModulePathWithPrefix = relativeModulePath.match(/^\.\./)
                    ? relativeModulePath
                    : `.${path.sep}${relativeModulePath}`;
                const moduleEntry = moduleByFileName[relativeModulePathWithPrefix];
                if (moduleEntry) {
                    moduleEntry.chunks.push(chunkId);
                }
                else {
                    moduleByFileName[relativeModulePathWithPrefix] = {
                        name: relativeModulePathWithPrefix,
                        size: moduleOriginalSize
                            ? moduleInfo.originalLength
                            : moduleInfo.renderedLength,
                        chunks: [chunkId],
                    };
                    sources.push(relativeModulePathWithPrefix, { fileName: modulePath, ...moduleInfo });
                }
            });
        }
        else if (entry.type === 'asset') {
            assets.push({
                name: entry.fileName,
                size: getByteSize(entry.source.toString()),
            });
            sources.push(entry.fileName, entry);
        }
        else ;
    });
    const stats = {
        builtAt: Date.now(),
        assets,
        chunks,
        modules: Object.values(moduleByFileName),
    };
    let result;
    try {
        result = transform(stats, sources, bundle);
    }
    catch (error) {
        console.error('Custom transform failed! Returning stats without any transforms.', error);
        result = stats;
    }
    return result;
};

export { bundleToWebpackStats, lookupChunkAsync };
//# sourceMappingURL=transform.mjs.map
